%

%
% Copyright (c) 2015 LAAS/CNRS
% All rights reserved.
%
% Redistribution  and  use  in  source  and binary  forms,  with  or  without
% modification, are permitted provided that the following conditions are met:
%
%   1. Redistributions of  source  code must retain the  above copyright
%      notice and this list of conditions.
%   2. Redistributions in binary form must reproduce the above copyright
%      notice and  this list of  conditions in the  documentation and/or
%      other materials provided with the distribution.
%
% THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
% WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
% MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
% ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
% WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
% ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
% IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
%
%                                           Anthony Mallet on Fri Jun 26 2015
%
function tk3_mask(block, noapply)
  values = get_param(block, 'MaskValues');
  names = get_param(block, 'MaskNames');
  enables = get_param(block, 'MaskEnables');
  vis = get_param(block, 'MaskVisibilities');

  user = get_param(block, 'UserData');
  user.param = struct();
  for p = 1:length(names)
    user.param.(names{p}) = values{p};
  end

  % parameters
  switch user.param.device
    case 'Flight controller'
      vis{idx('nmotor')} = 'on';
      enables{idx('nmotor')} = 'on';

      vis{idx('imu')} = 'on';
      vis{idx('imufreq')} = 'on';
      enables{idx('imu')} = 'on';
      enables{idx('imufreq')} = 'on';

      vis{idx('battery')} = 'on';
      vis{idx('batteryfreq')} = 'on';
      enables{idx('battery')} = 'on';
      enables{idx('batteryfreq')} = 'on';

    case 'Brushless controller'
      vis{idx('nmotor')} = 'off';
      enables{idx('nmotor')} = 'off';

      vis{idx('imu')} = 'off';
      vis{idx('imufreq')} = 'off';
      enables{idx('imu')} = 'off';
      enables{idx('imufreq')} = 'off';

      vis{idx('battery')} = 'off';
      vis{idx('batteryfreq')} = 'off';
      enables{idx('battery')} = 'off';
      enables{idx('batteryfreq')} = 'off';
  end

  % input/output port names
  mdisp = '';
  user.inports = 0;
  user.inidx = struct();
  user.inidx.start = -1;
  user.inidx.cmd = -1;

  user.outports = 0;
  user.outidx = struct();
  user.outidx.imu = -1;
  user.outidx.imuts = -1;
  user.outidx.mid = -1;
  user.outidx.vel = -1;
  user.outidx.pwm = -1;
  user.outidx.cur = -1;
  user.outidx.mts = -1;
  user.outidx.bat = -1;
  user.outidx.batts = -1;

  if strcmp(user.param.start, 'on')
    add_inlabel('start', 'start/stop');
  end
  switch user.param.device
    case 'Flight controller'
      if strcmp(user.param.velocities, 'on')
        add_inlabel('cmd', 'motor velocity []');
      end
      if strcmp(user.param.imu, 'on')
        add_outlabel('imu', 'imu [x, y, z, roll, pitch, yaw]');
        if strcmp(user.param.timestamps, 'on')
          add_outlabel('imuts', 'imu time');
        end
      end
      if strcmp(user.param.motor, 'on')
        add_outlabel('vel', 'motor velocity []');
        add_outlabel('pwm', 'motor pwm []');
        add_outlabel('cur', 'motor current []');
        if strcmp(user.param.timestamps, 'on')
          add_outlabel('mts', 'motor time []');
        end
      end
      if strcmp(user.param.battery, 'on')
        add_outlabel('bat', 'battery');
        if strcmp(user.param.timestamps, 'on')
          add_outlabel('batts', 'battery time');
        end
      end

    case 'Brushless controller'
      user.param.imu = 'off';
      if strcmp(user.param.velocities, 'on')
        add_inlabel('cmd', 'motor velocity');
      end
      add_outlabel('mid', 'motor id');
      if strcmp(user.param.motor, 'on')
        add_outlabel('vel', 'motor velocity');
        add_outlabel('pwm', 'motor pwm');
        add_outlabel('cur', 'motor current');
        if strcmp(user.param.timestamps, 'on')
          add_outlabel('mts', 'motor time');
        end
      end
      user.param.battery = 'off';
  end

  if nargin == 1
    set_param(block, ...
              'MaskDisplay', mdisp, ...
              'MaskVisibilities', vis, ...
              'MaskEnables', enables);
  end
  set_param(block, 'UserData', user);

  function index = idx(key)
    index = find(strcmp(names, key));
  end

  function add_inlabel(field, text)
    user.inports = user.inports+1;
    user.inidx.(field) = user.inports;
    mdisp = [mdisp sprintf('port_label(''input'',%d,''%s'')\n', ...
                           user.inports, text)];
  end
  function add_outlabel(field, text)
    user.outports = user.outports+1;
    user.outidx.(field) = user.outports;
    mdisp = [mdisp sprintf('port_label(''output'',%d,''%s'')\n', ...
                           user.outports, text)];
  end
end
